#ifndef _NEVIN_BEZIER_
#define _NEVIN_BEZIER_

#include "curve.h"

enum Continuity {G0, G1, C1};			// enumerated type to represent levels of continuity.

class bezier : public curve
{   public:
	// Constructors:  (no destructor, as the parent is sufficient.)
	bezier();
	bezier(unsigned int);
	bezier(unsigned int, GLfloat*);
	
	// Functions to read values from protected data:
	Continuity getContinuity() const;			// Access continuity of curve
	virtual int howManySegments() const;		// Returns number of curve segments

	// Functions to change values of protected data:
	virtual void set(unsigned int, coordEnum, GLfloat);	// Set single X, Y, or Z value of point
	virtual void set(unsigned int, GLfloat *);		// Set X, Y, and Z of a point
	virtual void setAll(GLfloat*);				// Set values of all points
	virtual void setAll(unsigned int, GLfloat*);		// Set number of points and values
	virtual void addPoint();					// Add a point to the end of the curve
	virtual void addPoint(GLfloat*);				// Add a point at the specified location
	virtual void setContinuity(Continuity);		// Set continuity of curve
	
	// Functions which provide information about the specified curve:
	virtual GLfloat getAtTimeT(coordEnum, GLfloat);	
	virtual void getAtTimeT(GLfloat, GLfloat*);		// Get value at specific time.
	virtual void draw();						// Draw the curve				
	
   protected:
	virtual void enforceCont(unsigned int, int);	// Move points to enforce continuity
	Continuity cont;							// continuity of join
};

#endif