// Generic curve functions and a general curve class for specific curves to inherit

#ifndef _NEVIN_GENCURVE_
#define _NEVIN_GENCURVE_
 
#include <stdlib.h>
#include <iostream>
#include <cmath>
//#include <GL/glut.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>  
#include "matrix.h"
using namespace std;

void multStuff(GLfloat M[4][4], GLfloat*, GLfloat*);
float dist(int, int, int, int);
GLfloat dist(GLfloat*, GLfloat*);
void cubicCurve(GLfloat*, GLfloat*, GLfloat*, GLint);

enum coordEnum{X = 0, Y, Z};

//typedef GLfloat matrix[4][4];
typedef GLfloat coordinate[3];
//typedef GLfloat coefficients[4];

 class curve
 {   public:
	// Constructor and Destructor functions:
	curve();						// Generic constructor
	~curve();						// Destructor
	
	// Functions to read values from protected data:
	virtual GLfloat get(unsigned int, coordEnum) const;	// Get a single X, Y, or Z location
	virtual void get(unsigned int, GLfloat*) const;		// Get a coordinate (X, Y, and Z)
	int howManyPoints() const;					// How many control points are there.
	virtual int howManySegments() const = 0;			// # of segments.  Pure Virtual.
	bool displayingPoints() const;					// Are the control points displayed?
	int getDetail() const;							// Detail curve is drawn with
	
	// Functions to change values of protected data:
	virtual void set(unsigned int, coordEnum, GLfloat);	// Set a single X, Y, or Z value
	virtual void set(unsigned int, GLfloat*);			// Set X, Y, and Z aspects of a coordinate
	virtual void setAll(GLfloat*);					// Set values of control points.
	virtual void setAll(unsigned int, GLfloat*);			// Set # of control points and their values
	virtual void addPoint();						// Add point to end of curve, on origin
	virtual void addPoint(GLfloat*);					// Add point to end, specify its position
	void displayPoints(bool);						// Choose if control points are shown
	virtual void setDetail(int);						// Change detail of drawn curves (this
											//   IS virtual, unlike its get* counterpart.
											//   Some curves change detail differently.)
	
	// Functions which provide information about the specified curve:
	// (These are pure virtual and must be implemented by children.)
	virtual GLfloat getAtTimeT(coordEnum, GLfloat) = 0;	
	virtual void getAtTimeT(GLfloat, GLfloat*) = 0;		// Get value at specific time.
	virtual void draw() = 0;						// Draw the curve				

   protected:
	virtual void calcCoeffs();						// Determine the coefficients to the cubic
											// parametric equation(s).
	
	// Data:
	basis * M;									// Basis matrix (or matrices).
	coordinate * points;							// Control points
	coefficient * coeffs[3];						// X, Y, and Z coefficients to equations
	int number;								// Number of control points
	bool showPoints;							// Display the control pts?
	int detail;									// Usually represents # of line segments
	bool dirty;									// When true, this signifies that the properties
											// of the curve have been altered and the
											// coefficients must be recalculated before
											// the curve is drawn.
};


#endif